@@ -1,16 +1,21 @@
-"""2D square lattice.
+r"""2D square lattice.
+
+The dispersion of the 2D square lattice is given by
+
+.. math:: ϵ_{k_x, k_y} = 2t [\cos(k_x) + \cos(k_y)]
+
+which takes values in :math:`ϵ_{k_x, k_y} ∈ [-4t, +4t] = [-D, +D]`.
 
 :half_bandwidth: The half_bandwidth corresponds to a nearest neighbor hopping
                  of `t=D/4`
 
 """
-from functools import partial
-
 import numpy as np
-from scipy.special import ellipkm1
-from mpmath import fp
 
-ellipk_z = partial(fp.ellipf, np.pi/2)
+from mpmath import mp
+from scipy.special import ellipkm1
+
+from gftool._util import _u_ellipk
 
 
 def gf_z(z, half_bandwidth):
@@ -24,7 +29,7 @@ def gf_z(z, half_bandwidth):
 
     Parameters
     ----------
-    z : complex ndarray or complex
+    z : complex np.ndarray or complex
         Green's function is evaluated at complex frequency `z`.
     half_bandwidth : float
         Half-bandwidth of the DOS of the square lattice.
@@ -32,7 +37,7 @@ def gf_z(z, half_bandwidth):
 
     Returns
     -------
-    gf_z : complex ndarray or complex
+    gf_z : complex np.ndarray or complex
         Value of the square lattice Green's function
 
     References
@@ -46,23 +51,18 @@ def gf_z(z, half_bandwidth):
     >>> gf_ww = gt.lattice.square.gf_z(ww, half_bandwidth=1)
 
     >>> import matplotlib.pyplot as plt
+    >>> _ = plt.axhline(0, color='black', linewidth=0.8)
     >>> _ = plt.plot(ww, gf_ww.real, label=r"$\Re G$")
     >>> _ = plt.plot(ww, gf_ww.imag, '--', label=r"$\Im G$")
+    >>> _ = plt.ylabel(r"$G*D$")
     >>> _ = plt.xlabel(r"$\omega/D$")
-    >>> _ = plt.axhline(0, color='black', linewidth=0.8)
-    >>> _ = plt.axvline(0, color='black', linewidth=0.8)
     >>> _ = plt.xlim(left=ww.min(), right=ww.max())
     >>> _ = plt.legend()
-    >>> plt.grid()
     >>> plt.show()
 
     """
     z_rel_inv = half_bandwidth/z
-    elliptic = np.frompyfunc(ellipk_z, 1, 1)(z_rel_inv**2)
-    try:
-        elliptic = elliptic.astype(np.complex)
-    except AttributeError:  # elliptic no array, thus no conversion necessary
-        pass
+    elliptic = _u_ellipk(z_rel_inv**2)
     gf_z = 2./np.pi/half_bandwidth*z_rel_inv*elliptic
     return gf_z
 
@@ -72,30 +72,28 @@ def hilbert_transform(xi, half_bandwidth):
 
     The Hilbert transform is defined
 
-    .. math::
-        \tilde{D}(ξ) = ∫_{-∞}^{∞}dϵ \frac{DOS(ϵ)}{ξ − ϵ}
+    .. math:: \tilde{D}(ξ) = ∫_{-∞}^{∞}dϵ \frac{DOS(ϵ)}{ξ − ϵ}
 
     The lattice Hilbert transform is the same as the non-interacting Green's
     function.
 
     Parameters
     ----------
-    xi : complex ndarray or complex
+    xi : complex np.ndarray or complex
         Point at which the Hilbert transform is evaluated
     half_bandwidth : float
         half-bandwidth of the DOS of the 2D square lattice
 
     Returns
     -------
-    hilbert_transform : complex ndarray or complex
+    hilbert_transform : complex np.ndarray or complex
         Hilbert transform of `xi`.
 
     Notes
     -----
     Relation between nearest neighbor hopping `t` and half-bandwidth `D`
 
-    .. math::
-        4t = D
+    .. math:: 4t = D
 
     See Also
     --------
@@ -108,9 +106,11 @@ def hilbert_transform(xi, half_bandwidth):
 def dos(eps, half_bandwidth):
     r"""DOS of non-interacting 2D square lattice.
 
+    Has a van Hove singularity (logarithmic divergence) at `eps = 0`.
+
     Parameters
     ----------
-    eps : float ndarray or float
+    eps : float np.ndarray or float
         DOS is evaluated at points `eps`.
     half_bandwidth : float
         Half-bandwidth of the DOS, DOS(| `eps` | > `half_bandwidth`) = 0.
@@ -118,9 +118,18 @@ def dos(eps, half_bandwidth):
 
     Returns
     -------
-    dos : float ndarray or float
+    dos : float np.ndarray or float
         The value of the DOS.
 
+    See Also
+    --------
+    gftool.lattice.square.dos_mp : multi-precision version suitable for integration
+
+    References
+    ----------
+    .. [economou2006] Economou, E. N. Green's Functions in Quantum Physics.
+       Springer, 2006.
+
     Examples
     --------
     >>> eps = np.linspace(-1.1, 1.1, num=500)
@@ -130,10 +139,9 @@ def dos(eps, half_bandwidth):
     >>> _ = plt.plot(eps, dos)
     >>> _ = plt.xlabel(r"$\epsilon/D$")
     >>> _ = plt.ylabel(r"DOS * $D$")
-    >>> _ = plt.axhline(0, color='black', linewidth=0.8)
     >>> _ = plt.axvline(0, color='black', linewidth=0.8)
+    >>> _ = plt.ylim(bottom=0)
     >>> _ = plt.xlim(left=eps.min(), right=eps.max())
-    >>> plt.grid()
     >>> plt.show()
 
     """
@@ -146,11 +154,22 @@ def dos(eps, half_bandwidth):
 
 
 # ∫dϵ ϵ^m DOS(ϵ) for half-bandwidth D=1
-# from: wolframalpha, do integral in python to assert accuracy
+# from: integral of dos_mp with mp.workdps(100)
+# for m in range(0, 22, 2):
+#     with mp.workdps(100):
+#         print(mp.quad(lambda eps: 2 * eps**m * dos_mp(eps), [0, 1])
+# rational numbers obtained by mp.identify
 dos_moment_coefficients = {
     2: 0.25,
-    4: 0.140625,
-    6: 0.0976563,
+    4: 9/64,
+    6: 25/256,
+    8: (35/128)**2,
+    10: (63/256)**2,
+    12: 0.0508890151977539,
+    14: 0.0438787937164307,
+    16: 0.0385653460398316,
+    18: 0.0343993364367634,
+    20: 0.031045401134179,
 }
 
 
@@ -189,6 +208,68 @@ def dos_moment(m, half_bandwidth):
         raise NotImplementedError('Calculation of arbitrary moments not implemented.') from keyerr
 
 
+def dos_mp(eps, half_bandwidth=1):
+    r"""Multi-precision DOS of non-interacting 2D square lattice.
+
+    Has a van Hove singularity (logarithmic divergence) at `eps = 0`.
+
+    This function is particularity suited to calculate integrals of the form
+    :math:`∫dϵ DOS(ϵ)f(ϵ)`. If you have problems with the convergence,
+    consider using :math:`∫dϵ DOS(ϵ)[f(ϵ)-f(0)] + f(0)` to avoid the singularity.
+
+    Parameters
+    ----------
+    eps : mpmath.mpf or mpf_like
+        DOS is evaluated at points `eps`.
+    half_bandwidth : mpmath.mpf or mpf_like
+        Half-bandwidth of the DOS, DOS(| `eps` | > `half_bandwidth`) = 0.
+        The `half_bandwidth` corresponds to the nearest neighbor hopping `t=D/4`
+
+    Returns
+    -------
+    dos_mp : mpmath.mpf
+        The value of the DOS.
+
+    See Also
+    --------
+    gftool.lattice.square.dos : vectorized version suitable for array evaluations
+
+    References
+    ----------
+    .. [economou2006] Economou, E. N. Green's Functions in Quantum Physics.
+       Springer, 2006.
+
+    Examples
+    --------
+    Calculate integrals:
+
+    >>> from mpmath import mp
+    >>> mp.quad(gt.lattice.square.dos_mp, [-1, 0, 1])
+    mpf('1.0')
+
+    >>> eps = np.linspace(-1.1, 1.1, num=500)
+    >>> dos_mp = [gt.lattice.square.dos_mp(ee, half_bandwidth=1) for ee in eps]
+    >>> dos_mp = np.array(dos_mp, dtype=np.float64)
+
+    >>> import matplotlib.pyplot as plt
+    >>> _ = plt.plot(eps, dos_mp)
+    >>> _ = plt.xlabel(r"$\epsilon/D$")
+    >>> _ = plt.ylabel(r"DOS * $D$")
+    >>> _ = plt.axvline(0, color='black', linewidth=0.8)
+    >>> _ = plt.ylim(bottom=0)
+    >>> _ = plt.xlim(left=eps.min(), right=eps.max())
+    >>> plt.show()
+
+    """
+    eps, half_bandwidth = mp.mpf(eps), mp.mpf(half_bandwidth)
+    if mp.fabs(eps) > half_bandwidth:
+        return mp.mpf('0')
+    # around 0 we have to double precision for `1 - eps**2` to resolve around singularity
+    with mp.workdps(mp.dps*2, normalize_output=True):
+        mm = -mp.powm1(eps / half_bandwidth, mp.mpf('2'))
+        return 2 / (mp.pi**2 * half_bandwidth) * mp.ellipk(mm)
+
+
 def stress_trafo(xi, half_bandwidth):
     r"""Single pole integration over the stress tensor function.
 
