@@ -1,6 +1,8 @@
 """Implementation of Padé based on robust pole finding.
 
-Check scaling: 3.5.2, replace 2-norm by infty norm."""
+Check scaling: 3.5.2, replace 2-norm by infty norm.
+
+"""
 from typing import NamedTuple
 
 import numpy as np
@@ -47,7 +49,7 @@ class RatPol(NamedTuple):
         numbers = np.arange(poles.size)
 
         for idx in range(poles.size):
-            # add expanded numinator
+            # add expanded numerator
             numer += resid[idx]*Polynomial.fromroots(poles[numbers != idx])
         return cls(numer, denom)
 
@@ -314,8 +316,12 @@ def residues_ols_tau_b(tau, fct_tau, poles, beta, weight=False):
 
 def residues_hybr_ols(z, fct_z, tau, fct_tau, poles, beta, moments=(), weight=False, weight_ratio=[.5, .5]):
     """Calculate residues fitting Matsubara as well as imaginary time data."""
+    EXPAND = 2**6
     polematrix_z = 1./np.subtract.outer(z, poles)
-    polematrix_tau = gt.pole_gf_tau(tau[:, newaxis], weights=1, poles=poles[:, newaxis], beta=beta)
+    iws = gt.matsubara_frequencies(range(z.size * EXPAND), beta=beta)
+    gf_iw = gt.pole_gf_z(iws, poles=poles[:, newaxis, newaxis], weights=1)
+    polematrix_tau = gt.fourier.iw2tau(gf_iw, beta=beta)[:, ::EXPAND].T
+    # polematrix_tau = gt.pole_gf_tau(tau[:, newaxis], weights=1, poles=poles[:, newaxis], beta=beta)
     polemat = np.concatenate((polematrix_z, polematrix_tau), axis=-2)
     funcvec = np.concatenate((fct_z, fct_tau), axis=-1)
     scaling = 1./np.linalg.norm(polemat, axis=-1, keepdims=True)
